==> saga_2.0.4+dfsg.orig.tar.gz: saga-2.0/src/modules_terrain_analysis/terrain_analysis/ta_hydrology/LakeFlood.cpp <==

///////////////////////////////////////////////////////////
//                                                       //
//                         SAGA                          //
//                                                       //
//      System for Automated Geoscientific Analyses      //
//                                                       //
//                    Module Library:                    //
//                     ta_hydrology                      //
//                                                       //
//-------------------------------------------------------//
//                                                       //
//                     LakeFlood.cpp                     //
//                                                       //
//                 Copyright (C) 2005 by                 //
//                    Volker Wichmann                    //
//                                                       //
//-------------------------------------------------------//
//                                                       //
// This file is part of 'SAGA - System for Automated     //
// Geoscientific Analyses'. SAGA is free software; you   //
// can redistribute it and/or modify it under the terms  //
// of the GNU General Public License as published by the //
// Free Software Foundation; version 2 of the License.   //
//                                                       //
// SAGA is distributed in the hope that it will be       //
// useful, but WITHOUT ANY WARRANTY; without even the    //
// implied warranty of MERCHANTABILITY or FITNESS FOR A  //
// PARTICULAR PURPOSE. See the GNU General Public        //
// License for more details.                             //
//                                                       //
// You should have received a copy of the GNU General    //
// Public License along with this program; if not,       //
// write to the Free Software Foundation, Inc.,          //
// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
// USA.                                                  //
//                                                       //
//-------------------------------------------------------//
//                                                       //
//    e-mail:     reklovw@web.de                         //
//                                                       //
///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////
//														 //
//														 //
//														 //
///////////////////////////////////////////////////////////

//---------------------------------------------------------
#include "LakeFlood.h"

///////////////////////////////////////////////////////////
//														 //
//				Construction/Destruction				 //
//														 //
///////////////////////////////////////////////////////////

CLakeFlood::CLakeFlood(void)
{
	// Parameter Settings for input dialog of the module

	Parameters.Set_Name(_TL("Lake Flood"));
	Set_Author(_TL("Copyrights (c) 2005 by Volker Wichmann"));
	Parameters.Set_Description(_TW(
		"This module can be used to simulate the extent and volume of a lake for a specified water depth in a seed cell.\r\n"
	));


	Parameters.Add_Grid(	
		NULL, "ELEV", _TL("DEM"), 
		_TL("digital elevation model [m]"),
		PARAMETER_INPUT
	);
	Parameters.Add_Grid(	
		NULL, "INDEPTH", _TL("Water Depth"),
		_TL("seed cell coded with local water depth [m], all other cells NoData"),
		PARAMETER_INPUT
	);
	Parameters.Add_Grid(	
		NULL, "OUTDEPTH", _TL("Lake"), 
		_TL("extent of lake, coded with water depth [m]"),
		PARAMETER_OUTPUT
	);

}

//---------------------------------------------------------
CLakeFlood::~CLakeFlood(void)
{}


///////////////////////////////////////////////////////////
//														 //
//														 //
//														 //
///////////////////////////////////////////////////////////


bool CLakeFlood::On_Execute(void)		
{
	CSG_Grid	*pElev, *pIdepth, *pOdepth;
	CTraceOrder	*newCell, *firstCell, *iterCell, *lastCell;
	int			x, y, ix, iy, i;
	double		wzSeed, z, d;

	pElev		= Parameters("ELEV")->asGrid();
	pIdepth		= Parameters("INDEPTH")->asGrid();
	pOdepth		= Parameters("OUTDEPTH")->asGrid();


	// Initialize Grids
	pOdepth->Assign(0.0);

	
	// Get seed cell and water depth
	for(y=0; y<Get_NY(); y++)									
	{
		for(x=0; x<Get_NX(); x++)
		{
			if( !pIdepth->is_NoData(x, y) )
			{
				if( !pElev->is_NoData(x, y) )
				{
					newCell = new CTraceOrder();
					newCell->x = x;
					newCell->y = y;
					firstCell = newCell;
					wzSeed = pIdepth->asDouble(x, y);
					pOdepth->Set_Value(x, y, wzSeed);
					wzSeed +=  pElev->asDouble(x, y);
					// terminate search
				}
				else
					firstCell = NULL;
			}
		}
	}

	// main
	//-----------------------------------------------------
	iterCell = firstCell;
	lastCell = firstCell;

	while( iterCell != NULL ) 
	{
		x	= iterCell->x;
		y	= iterCell->y;

		z		= pElev->asDouble(x, y);

		for(i=0; i<8; i++)												
		{
			ix	= Get_xTo(i, x);			
			iy	= Get_yTo(i, y);			
						
			if(	is_InGrid(ix, iy) && !pElev->is_NoData(ix, iy) && pOdepth->asDouble(ix, iy) == 0.0 )
			{ 
				d	= pElev->asDouble(ix, iy);
				if( d < wzSeed )
				{
					pOdepth->Set_Value(ix, iy, (wzSeed - d));
					newCell = new CTraceOrder();
					newCell->x = ix;
					newCell->y = iy;
					newCell->prev = lastCell;
					lastCell->next = newCell;
					lastCell = newCell;
				}
			}
		}
	
		newCell = firstCell;

		if( newCell->next == NULL )
		{
			firstCell = NULL;
			delete (newCell);
		}
		else
		{
			newCell->next->prev = NULL;
			firstCell = newCell->next;
			newCell->next = NULL;
			delete (newCell);
		}

		iterCell = firstCell;
	}

	//-----------------------------------------------------

	return( true );
}

==> saga_2.1.0+dfsg.orig.tar.gz: saga-2.1.0+dfsg/src/modules_terrain_analysis/terrain_analysis/ta_hydrology/LakeFlood.cpp <==
/**********************************************************
 * Version $Id: LakeFlood.cpp 911 2011-02-14 16:38:15Z reklov_w $
 *********************************************************/

///////////////////////////////////////////////////////////
//                                                       //
//                         SAGA                          //
//                                                       //
//      System for Automated Geoscientific Analyses      //
//                                                       //
//                    Module Library:                    //
//                     ta_hydrology                      //
//                                                       //
//-------------------------------------------------------//
//                                                       //
//                     LakeFlood.cpp                     //
//                                                       //
//                 Copyright (C) 2005 by                 //
//                    Volker Wichmann                    //
//                                                       //
//-------------------------------------------------------//
//                                                       //
// This file is part of 'SAGA - System for Automated     //
// Geoscientific Analyses'. SAGA is free software; you   //
// can redistribute it and/or modify it under the terms  //
// of the GNU General Public License as published by the //
// Free Software Foundation; version 2 of the License.   //
//                                                       //
// SAGA is distributed in the hope that it will be       //
// useful, but WITHOUT ANY WARRANTY; without even the    //
// implied warranty of MERCHANTABILITY or FITNESS FOR A  //
// PARTICULAR PURPOSE. See the GNU General Public        //
// License for more details.                             //
//                                                       //
// You should have received a copy of the GNU General    //
// Public License along with this program; if not,       //
// write to the Free Software Foundation, Inc.,          //
// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
// USA.                                                  //
//                                                       //
//-------------------------------------------------------//
//                                                       //
//    e-mail:     reklovw@web.de                         //
//                                                       //
///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////
//														 //
//														 //
//														 //
///////////////////////////////////////////////////////////

//---------------------------------------------------------
#include "LakeFlood.h"

///////////////////////////////////////////////////////////
//														 //
//				Construction/Destruction				 //
//														 //
///////////////////////////////////////////////////////////

CLakeFlood::CLakeFlood(void)
{

	Set_Name		(_TL("Lake Flood"));
	Set_Author		(SG_T("Volker Wichmann (c) 2005-2010"));
	Set_Description	(_TW("This module can be used to flood a digital elevation model from seed points. "
						"Seed points have to be coded either with local water depth or absolute water level.\r\n"
	));


	Parameters.Add_Grid(	
		NULL, "ELEV", _TL("DEM"), 
		_TL("digital elevation model"),
		PARAMETER_INPUT
	);
	Parameters.Add_Grid(	
		NULL, "SEEDS", _TL("Seeds"),
		_TL("seed cells coded with local water depth or absolute water level, all other cells NoData"),
		PARAMETER_INPUT
	);
	Parameters.Add_Value(
		NULL, "LEVEL", _TL("Absolute Water Levels"),
		_TL("check this in case seed cells are coded with absolute water level"),
		PARAMETER_TYPE_Bool, false
	);
	Parameters.Add_Grid(	
		NULL, "OUTDEPTH", _TL("Lake"), 
		_TL("extent of lake, coded with water depth"),
		PARAMETER_OUTPUT
	);
	Parameters.Add_Grid(	
		NULL, "OUTLEVEL", _TL("Surface"), 
		_TL("Flooded digital elevation model"),
		PARAMETER_OUTPUT
	);

}

//---------------------------------------------------------
CLakeFlood::~CLakeFlood(void)
{}


///////////////////////////////////////////////////////////
//														 //
//														 //
//														 //
///////////////////////////////////////////////////////////

//---------------------------------------------------------
bool CLakeFlood::On_Execute(void)		
{
	CSG_Grid	*pElev, *pSeeds, *pOdepth, *pOlevel;
	CTraceOrder	*newCell, *firstCell, *iterCell, *lastCell;
	int			x, y, ix, iy, i;
	bool		bLevel;
	SEED		thisSeed;
	std::list<SEED>				seeds;
	std::list<SEED>::iterator	it;


	//---------------------------------------------------------
	pElev		= Parameters("ELEV")->asGrid();
	pSeeds		= Parameters("SEEDS")->asGrid();
	bLevel		= Parameters("LEVEL")->asBool();
	pOdepth		= Parameters("OUTDEPTH")->asGrid();
	pOlevel		= Parameters("OUTLEVEL")->asGrid();



	//---------------------------------------------------------
	// Initialize

	pOdepth->Assign(0.0);
	pOlevel->Assign(pElev);

	for( y=0; y<Get_NY(); y++ )									
	{
		for( x=0; x<Get_NX(); x++ )
		{
			if( !pSeeds->is_NoData(x, y) && !pElev->is_NoData(x, y) )
			{
				thisSeed.x	= x;
				thisSeed.y	= y;
				if( bLevel )
					thisSeed.z	= pSeeds->asDouble(x, y);
				else
					thisSeed.z	= pSeeds->asDouble(x, y) + pElev->asDouble(x, y);
				seeds.push_back(thisSeed);
			}
		}
	}


	//-----------------------------------------------------
	// Flood

	for( it=seeds.begin(); it!=seeds.end(); it++ )
	{
		if( it->z <= pOlevel->asDouble(it->x, it->y) )
			continue;

		newCell		= new CTraceOrder();
		newCell->x	= it->x;
		newCell->y	= it->y;
		firstCell	= newCell;

		pOdepth->Set_Value(it->x, it->y, it->z - pElev->asDouble(it->x, it->y));
		pOlevel->Set_Value(it->x, it->y, it->z);


		iterCell = firstCell;
		lastCell = firstCell;

		while( iterCell != NULL ) 
		{
			x	= iterCell->x;
			y	= iterCell->y;

			for( i=0; i<8; i++ )												
			{
				ix	= Get_xTo(i, x);			
				iy	= Get_yTo(i, y);			
							
				if(	is_InGrid(ix, iy) && !pElev->is_NoData(ix, iy) && pOlevel->asDouble(ix, iy) < it->z )
				{ 
					pOdepth->Set_Value(ix, iy, it->z - pElev->asDouble(ix, iy));
					pOlevel->Set_Value(ix, iy, it->z);
					newCell = new CTraceOrder();
					newCell->x = ix;
					newCell->y = iy;
					newCell->prev = lastCell;
					lastCell->next = newCell;
					lastCell = newCell;
				}
			}
		
			newCell = firstCell;

			if( newCell->next == NULL )
			{
				firstCell = lastCell = NULL;
				delete (newCell);
				newCell = NULL;
			}
			else
			{
				newCell->next->prev = NULL;
				firstCell = newCell->next;
				newCell->next = NULL;
				delete (newCell);
				newCell = NULL;
			}

			iterCell = firstCell;
		}
	}


	//-----------------------------------------------------
	return( true );
}
